// myProfilerToFile.ijm
// Julien Berro, Pollard lab, Yale University
// 4 November 2012
// Generates a montage with individual cells and generates a file with intensity profile information (projected on the long axis)
// 1. Open a file that contains an image, a stack or an hyperstack
// 2. Circle the cells and add each one in the ROI Manager (alternatively, open a roiset zip file)
// 3. run this macro
// 4. A montage with all the cells realigned on their long axis is made and saved as MontageAllCells.tif in the directory of the initial movie (if makeMontageCellProfile==true)
// 5. A file containing for each cell its diameter, its intensity profile, its intensity profile per unit area vs position, at the current frame, slice and channel, is generated and saved as CellProfiles.xls in the directory of the initial movie ( if measureProfiles==true) 


// Modifiable variables
multiplyBackgroundBy=0; // set to 1 to keep background, 0 to remove background outside the cells (0 for profile measurements). Set to something between 0 and 1 to attenuate background. WARNING: if the value is not 0, the diameters and the intensity per unit area won't be estimated properly
makeMontageCellProfile=true; // true if you want to generate and save the montage with individual cells aligned on their long axis
measureProfiles=true; // true if you want a file with the profiles to be generated
lineWidth=55; // 55: Width of the box that will contain the cells. 55 for entire pombe  
fitEllipseBeforeProfile=true; // true if you want to align your cells according to their fitting ellipse. Otherwise the cell is aligned along its Feret axis. (Fitting with an ellipse gives better result in general) 
nbRoisForOneProfile=1; // number of consecutive rois that have information for the same element. 1 in most cases (cellProfiles...); 2 for patches (first and last positions); 3 for patches that will be reoriented (same as 2 plus one extra roi that should be a straight line that represents the membrane)
// Parameters valid only if  nbRoisForOneProfile==3
drawMembrane=true; // Only valid if  nbRoisForOneProfile==3. True if the third roi (typically the membrane should be printed on the image) 
resizeFactor=1; // PROBLEM NEEDS TO BE FIXED // Only valid if  nbRoisForOneProfile==3. Ratio used to resize the montage images (0<resizeFactor<1 for reduction, resizeFactor>1resizeFactor, resizeFactor==1 for no resizing)
coloredFinalMontage=false; // Only valid if  nbRoisForOneProfile==3. Finale image is a RGB image
//keepSliceInfoRoiForMontageAndProfiles=false;	// true to use only the slice specified in the roi to make the montage and the profile (default: false)
keepSliceInfoRoiForMontageAndProfiles=false; //default: false

estimateBackgrounds=false;

extendedName="CellName_";

// Default parameters 
 reduceRoisInPX=0; //default 0
 projectMin=-1; //default -1
 projectMax=-1; // default -1
 xlsFileName0="CellProfiles"; //Default
 montageFileName0="Montage"; //Default
  reducedRoisFileName0="";

measureAngleMembrane=true;
txtAngleToSave="";

numberOfColumnsForMontage=1;


Dialog.create("Profiler parameters");
if (multiplyBackgroundBy==1) {defaultRemoveBg=false;}
else {defaultRemoveBg=true;}
Dialog.addCheckbox("Make montage", makeMontageCellProfile);
txt1Roi="Cells (1 roi/cell)";
txt2Rois="Patches (2 rois/patch)";
txt3Rois="Patches with membrane (3 rois/patch)";
txt4Rois="Cell background (1 roi/cell)";
Dialog.addChoice("Montage type", newArray(txt1Roi,txt2Rois,txt3Rois,txt4Rois));
Dialog.addCheckbox("Remove background around Roi", defaultRemoveBg);
Dialog.addNumber("Number of columns (-1 for squared montage)",numberOfColumnsForMontage);
Dialog.addNumber("Width of individual cell (in px)", lineWidth);
Dialog.addCheckbox("Measure profiles along ROIs long axis", measureProfiles);
Dialog.addCheckbox("Keep only current ROI slice and frame for montage", keepSliceInfoRoiForMontageAndProfiles);
Dialog.addString("Output file name", extendedName) ;
Dialog.addCheckbox("Draw membrane (only valid with 3 rois/patch)", drawMembrane);
Dialog.show();	

makeMontageCellProfile=Dialog.getCheckbox();
montageType=Dialog.getChoice();
defaultRemoveBg=Dialog.getCheckbox(); 
numberOfColumnsForMontage=Dialog.getNumber();
lineWidth=Dialog.getNumber();
measureProfiles=Dialog.getCheckbox();
keepSliceInfoRoiForMontageAndProfiles=Dialog.getCheckbox();
extendedName=Dialog.getString();
drawMembrane=Dialog.getCheckbox();

if (montageType==txt1Roi) {nbRoisForOneProfile=1;}
else if (montageType==txt2Rois) {nbRoisForOneProfile=2;}
else if (montageType==txt3Rois) {nbRoisForOneProfile=3;}
else if (montageType==txt4Rois) {nbRoisForOneProfile=1; estimateBackgrounds=true;}

if (defaultRemoveBg) {multiplyBackgroundBy=0;}
else {multiplyBackgroundBy=1;}



if (estimateBackgrounds){
//// Parameters for background analysis
reduceRoisInPX=-14; //The initial Roi will be expanded or shrunk by reduceRoisInPX pixels. default 0; -14 for cytoplasm 
projectMin=9; //first z-slice for the projection to measure profiles;  default -1 to project all 
projectMax=9; //last z-slice for the projection to measure profiles; default -1 to project all
xlsFileName0="CellBackgroundProfiles";//"CellProfiles";//"CellBackgroundProfiles";
montageFileName0="MontageAllBackgrounds";//"MontageAllCells";//"MontageAllBackgrounds";
reducedRoisFileName0="BackgroundRois";//"CellRois";//"BackgroundRois";
closeStack=true; // saves memory after projecting images by closing the original stack

Dialog.create("Background Profiler parameters");
Dialog.addNumber("Reduce Roi by (px)",reduceRoisInPX);
Dialog.addNumber("Bottom slice",projectMin);
Dialog.addNumber("Top slice",projectMax);
Dialog.addString("Output data file name", xlsFileName0) ;
Dialog.addString("Output montage file name", montageFileName0) ;
Dialog.addString("Output reduced Roiset file name", reducedRoisFileName0) ;
//Dialog.addCheckbox("",closeStack);
Dialog.show();	
reduceRoisInPX=Dialog.getNumber();
projectMin=Dialog.getNumber();
projectMax=Dialog.getNumber();
xlsFileName0=Dialog.getString();
montageFileName0=Dialog.getString();
reducedRoisFileName0=Dialog.getString();
// End of modifiable variables
}

// Extend Names
xlsFileName0=extendedName+xlsFileName0;
montageFileName0=extendedName+montageFileName0;
reducedRoisFileName0=extendedName+reducedRoisFileName0;


//Add parameter values to output names
getDimensions(mainWidth, mainHeight, mainChannelNb, mainSliceNb, mainFrameNb);
txtParameters="_T="+mainFrameNb+"_Z="+mainSliceNb+"_BGx"+multiplyBackgroundBy;
if (reduceRoisInPX>0){
	txtParameters=txtParameters+"_"+reduceRoisInPX+"px";
}
if (projectMin!=-1||projectMax!=-1){
	txtParameters=txtParameters+"_ProjFromSl="+projectMin;
	txtParameters=txtParameters+"_toSl="+projectMax;
}
xlsFileName=xlsFileName0+txtParameters+".xls";
montageFileName=montageFileName0+txtParameters+".tif";
reducedRoisFileName=reducedRoisFileName0+txtParameters+".zip";
anglesFileName="Angles.xls";

// Start
var xValues, diameters, profile, profilePerAreaUnit;
setBatchMode(true);
idSelectedImage=getImageID();
getPixelSize(unit, pixelWidth, pixelHeight);
imageDir=getDirectory("image");
getDimensions(mainWidth, mainHeight, mainChannelNb, mainSliceNb, mainFrameNb);
getDimensions(originalWidth, originalHeight, originalChannelNb, originalSliceNb, originalFrameNb);
Stack.getPosition(channel, slice, frame);
nRois=roiManager("count");
if (nRois==0){
	showMessage("Macro aborted: the Roi Manager should contain Rois!");
}
else if(floor(nRois/nbRoisForOneProfile)-nRois/nbRoisForOneProfile!=0){
	showMessage("Macro aborted: the number of rois should be a multiple of "+nbRoisForOneProfile+"\n Check your rois or change the parameter 'nbRoisForOneProfile'");
}
else{
	nProfiles=nRois/nbRoisForOneProfile;
	//Duplicate the image and project it if needed 
	if ((projectMin>0) && (projectMax<=originalSliceNb)){
		//roiManager("Deselect");
		run("Select None");
		run("Duplicate...", "title=temp.tif duplicate");
		idTmpImage=getImageID();
		run("Z Project...", "start="+projectMin+" stop="+projectMax+" projection=[Average Intensity] all");
		idOldImage=idSelectedImage;
		idSelectedImage=getImageID();
		getDimensions(mainWidth, mainHeight, mainChannelNb, mainSliceNb, mainFrameNb);
		//selectImage(idTmpImage);
		//close();
		outputRoiName=imageDir+reducedRoisFileName;
		if (closeStack){
			selectImage(idOldImage);
			close();
			selectImage(idSelectedImage);
		}
	}
	txt="";
	fileSep=File.separator;
	sp=split(imageDir,fileSep);
	folderName=sp[sp.length-1];
	outputFileName=imageDir+xlsFileName;
	outputMontageName=imageDir+montageFileName;
	idCellProfiles=newArray(nProfiles);
	nameCellProfiles=newArray(nProfiles);
	widthCellProfiles=newArray(nProfiles);
	heightCellProfiles=newArray(nProfiles);
	maxWidth=0;
	maxHeight=0;
	maxNbFrames=0;
	for (profileNb=0;profileNb<nProfiles;profileNb++){
		idSelectedRoi=profileNb*nbRoisForOneProfile;
		selectImage(idSelectedImage);
		roiManager("select",idSelectedRoi);
		if (reduceRoisInPX!=0){ // Reduces the ROI
			run("Enlarge...", "enlarge="+reduceRoisInPX+" pixel");
			roiManager("Update");
		}
		//waitForUser("aa");
		roiName=getInfo("selection.name");
		Stack.setPosition(channel, slice, frame);
		idCellProfiles[profileNb]=profileCell(idSelectedRoi, idSelectedImage, lineWidth, multiplyBackgroundBy, makeMontageCellProfile,measureProfiles,keepSliceInfoRoiForMontageAndProfiles);
		if (makeMontageCellProfile){
			selectImage(idCellProfiles[profileNb]);
			rename("CellProfile_"+idCellProfiles[profileNb]);
			nameCellProfiles[profileNb]=getTitle();
			getDimensions(tmpWidth, tmpHeight, tmpChannels, tmpSlices, tmpFrames);
			widthCellProfiles[profileNb]=tmpWidth;
			heightCellProfiles[profileNb]=tmpHeight;
			if (tmpWidth>maxWidth){
				maxWidth=tmpWidth;
			}
			if (tmpHeight>maxHeight){
				maxHeight=tmpHeight;
			}
			if (tmpFrames>maxNbFrames){
				maxNbFrames=tmpFrames;
			}
		}
		if (measureProfiles){
		txt=txt+array2Text(xValues,"xValues ("+roiName+")")+"\n";
		txt=txt+array2Text(diameters,"diameters ("+roiName+")")+"\n";
		txt=txt+array2Text(profile,"profile ("+roiName+")")+"\n";
		txt=txt+array2Text(profilePerAreaUnit,"profilePerAreaUnit ("+roiName+")")+"\n \n";
		}
	}
	if (makeMontageCellProfile){
		if (numberOfColumnsForMontage<=0){
			nbColumnFigure=ceil(sqrt(nProfiles));
		}
		else{
			nbColumnFigure=numberOfColumnsForMontage;
		}
		nbRowFigure=ceil(nProfiles/nbColumnFigure);
		
		idTempRowMontage=newArray(nbRowFigure);
		nameTempRowMontage=newArray(nbRowFigure);
		cpt=0;
		// Combining images in row montages 
		for (i=0;i<nbRowFigure;i++){
			combinedMontageID=idCellProfiles[0+i*nbColumnFigure];
			selectImage(combinedMontageID);
			run("Canvas Size...", "width="+maxWidth+" height="+maxHeight+" position=Center zero");
			cpt++;
			for (j=1;(j<nbColumnFigure&&cpt<nProfiles);j++){
				selectImage(combinedMontageID);
				CombinedMontageTitle=getTitle();
				selectImage(idCellProfiles[j+i*nbColumnFigure]);
				curMontageTitle=getTitle();
				run("Canvas Size...", "width="+maxWidth+" height="+maxHeight+" position=Center zero");
				//run("Combine...", "stack1=["+CombinedMontageTitle+"] stack2=["+curMontageTitle+"]");
				run("Stack Combiner", "stack1=["+CombinedMontageTitle+"] stack2=["+curMontageTitle+"]");
				combinedMontageID=getImageID();
				cpt++;
			}
			idTempRowMontage[i]=combinedMontageID;
			nameTempRowMontage[i]="MontageRow_"+i;
			rename(nameTempRowMontage[i]);
		}
		
		//Combines row montages into one montage
		combinedMontageID=idTempRowMontage[0];	
		for (i=1;i<nbRowFigure;i++){
			selectImage(combinedMontageID);
			CombinedMontageTitle=getTitle();
			selectImage(idTempRowMontage[i]);
			curMontageTitle=getTitle();
			run("Canvas Size...", "width="+(nbColumnFigure*maxWidth)+" height="+maxHeight+" position=Center-Left zero");
			//run("Combine...", "stack1=["+CombinedMontageTitle+"] stack2=["+curMontageTitle+"] combine");
			run("Stack Combiner", "stack1=["+CombinedMontageTitle+"] stack2=["+curMontageTitle+"] combine");
			combinedMontageID=getImageID();
		}
		rename("Final montage");
		if ((mainSliceNb>1||mainFrameNb>1)&&!Stack.isHyperstack&&!keepSliceInfoRoiForMontageAndProfiles){
			if (nbRoisForOneProfile==1){
				run("Stack to Hyperstack...", "order=xyczt(default) channels="+mainChannelNb+" slices="+mainSliceNb+" frames="+mainFrameNb+" display=Grayscale");
			}
			else{
				run("Stack to Hyperstack...", "order=xyczt(default) channels="+mainChannelNb+" slices="+mainSliceNb+" frames="+maxNbFrames+" display=Grayscale");
			}
			
			
		}
		save(outputMontageName);
		// Close the projection image if created and save reduced rois
	}
	if (projectMin>0 && projectMax<=originalSliceNb){
	//if (estimateBackgrounds){
		print("outputRoiName: ", outputRoiName);
		roiManager("Deselect");
		roiManager("Save", outputRoiName);
		print(outputRoiName+" saved successfully!");
		//wait(1000);
		selectImage(idSelectedImage);
		close();
		selectImage(combinedMontageID);
	}
	if (measureProfiles){
		f = File.open(outputFileName);
		print(f,txt);
		File.close(f);
		print(outputFileName+" saved successfully!");
	}
	if (nbRoisForOneProfile==3 && measureAngleMembrane==true){
		for (profileNb=0;profileNb<nProfiles;profileNb++){
			idSelectedRoi=profileNb*nbRoisForOneProfile+2;
			selectImage(idSelectedImage);
			roiManager("select",idSelectedRoi);
			List.setMeasurements;
			roisetName=call("ij.plugin.frame.RoiManager.getName", idSelectedRoi);
			angle = List.getValue("Angle");
			txtAngleToSave=txtAngleToSave+roisetName+"\t"+angle+"\n";
		}
		f = File.open(imageDir+anglesFileName);
		print(f,txtAngleToSave);
		File.close(f);
		print(imageDir+anglesFileName+" saved successfully!");
		//print(txtAngleToSave);
	}
	
	//}
	setBatchMode(false);
	setLineWidth(1);
	Stack.setPosition(channel, slice, frame);
}

function profileCell(idSelectedRoi, idSelectedImage, lineWidth, multiplyBackgroundBy, makeMontageCellProfile, measureProfiles,keepSliceInfoRoiForMontageAndProfiles){
	// Uses side effects on the global variables  'xValues', 'diameters', 'profile', 'profilePerAreaUnit'to return results
	// Uses global variables nbRoisForOneProfile, mainFrameNb;
	// keepSliceInfoRoiForMontageAndProfiles is true to use only the slice specified in the roi to make the montage and the profile
	extraPixels=1;
	selectImage(idSelectedImage);
	titleSelectedImage=getTitle();
	getDimensions(tmpWidth, tmpHeight, tmpChannels, tmpSlices, tmpFrames);
	nSlicesOriginalImage=nSlices;
	Stack.getPosition(channel, slice, frame);
	if (tmpFrames>1) stackLength=tmpFrames;
	else if (tmpSlices>1) stackLength=tmpSlices;
	else if (tmpChannels>1) stackLength=tmpChannels;
	else stackLength=tmpFrames;
	run("Select None");
	//run("Duplicate...", "title=temp.tif duplicate range=1-"+stackLength);
	//HERE!!!
	run("Duplicate...", "title=temp.tif duplicate");
	idCopy=getImageID();
	roiManager("select",idSelectedRoi);
	if (nbRoisForOneProfile>1){
		Stack.getPosition(channelDump, sliceDump, firstFrameMontage);
		getSelectionBounds(xFirst, yFirst, widthFirst, heightFirst); //x and y : coordinates of the upper left corner of the rectangle. 
		roiManager("select",idSelectedRoi+1);
		Stack.getPosition(channelDump, sliceDump, lastFrameMontage);
		getSelectionBounds(xSecond, ySecond, widthSecond, heightSecond);
		xMinRect=minOf(xFirst,xSecond)-extraPixels;
		yMinRect=minOf(yFirst,ySecond)-extraPixels;
		xMaxRect=maxOf(xFirst+widthFirst,xSecond+widthSecond)+extraPixels;
		yMaxRect=maxOf(yFirst+heightFirst,ySecond+heightSecond)+extraPixels;
		widthRect=xMaxRect-xMinRect;
		heightRect=yMaxRect-yMinRect;
		roiManager("select",idSelectedRoi);
		makeRectangle(xMinRect, yMinRect, widthRect, heightRect);
		roiManager("Update");
		//waitForUser("title", "msg");
	}
	//run("Fit Spline");
	run("Make Inverse");
	if (multiplyBackgroundBy!=1){
		run("Multiply...", "value="+multiplyBackgroundBy+" stack");
	}
	roiManager("select",idSelectedRoi);
	//Get Ellipse
	if (fitEllipseBeforeProfile){
		run("Fit Ellipse");
	}
	if (nbRoisForOneProfile==1){
		//Get Feret coordinates
		diameter = 0.0;
		getSelectionCoordinates(xCoordinates, yCoordinates);
		n = xCoordinates.length;
		for (i=0; i<n; i++) {
			for (j=i; j<n; j++) {
				dx = xCoordinates[i] - xCoordinates[j];
				dy = yCoordinates[i] - yCoordinates[j];
				d = sqrt(dx*dx + dy*dy);
				if (d>diameter) {
					diameter = d;
					i1 = i;
					i2 = j;
				}
			}
		}
		makeLine(xCoordinates[i1], yCoordinates[i1],xCoordinates[i2],yCoordinates[i2],lineWidth);
		if (keepSliceInfoRoiForMontageAndProfiles){
			run("Straighten...", "line="+lineWidth);
		}
		else{
			run("Straighten...", "process");
		}
	}
	else if(nbRoisForOneProfile>=2){
		angle=0;
		xCoordinates=0;
		yCoordinates=0;
		if (nbRoisForOneProfile==3){ //compute the rotation angle if membrane is given
			roiManager("select",idSelectedRoi+2);
			//if (drawMembrane){
			//	run("RGB Color");
			//	setColor(255, 0, 0);
			//	run("Fill", "stack");
			//	run("Draw","stack");
			//}
			getSelectionCoordinates(xCoordinates, yCoordinates);
			cosine=xCoordinates[1]-xCoordinates[0];
			sine=yCoordinates[1]-yCoordinates[0];
			norm=sqrt(cosine*cosine+sine*sine);
			cosine=cosine/norm;
			sine=sine/norm;
			angle=acos(cosine);
			sign=asin(sine);
			if (sign<0){
				angle=-angle;
			}
			angle=-angle*180/PI; // Aligns with x axis
		}
		
		roiManager("select",idSelectedRoi);
		nbPixToEnlarge=0; // only for nbRoisForOneProfile==3
		xBox=0; yBox=0; widthBox=0; heightBox=0;
		if (nbRoisForOneProfile==3){// Enlarge to prevent from blanks after rotation; shrink again after rotating
			// Calculate the best enlargement to prevent from blank areas in the montage after rotation
			getSelectionBounds(x0, y0, width0, height0);
			diagonal=round(sqrt(width0*width0+height0*height0));
			nbPixToEnlarge=round(maxOf((diagonal-width0)/2,(diagonal-height0)/2));
			//nbPixToEnlarge=2;
			run("Enlarge...", "enlarge="+nbPixToEnlarge+"  pixel");
			getSelectionBounds(xBox, yBox,widthBox, heightBox);
		}
		
		//print("first="+firstFrameMontage);
		//print("last="+lastFrameMontage);
		if (Stack.isHyperstack){
			run("Duplicate...", "title=["+titleSelectedImage+"] duplicate frames="+firstFrameMontage+"-"+lastFrameMontage);
		}
		else{
			run("Duplicate...", "title=["+titleSelectedImage+"] duplicate range="+firstFrameMontage+"-"+lastFrameMontage);
			run("Stack to Hyperstack...", "order=xyczt(default) channels=1 slices=1 frames="+(lastFrameMontage-firstFrameMontage+1)+" display=Color");
		}
		if (nbRoisForOneProfile==3){ //rotate to align horizontally the line from the 3rd Roi
			// Coordinates of the line in the cropped image
			newX0Line=xCoordinates[0]-xBox;
			newY0Line=yCoordinates[0]-yBox;
			newX1Line=xCoordinates[1]-xBox;
			newY1Line=yCoordinates[1]-yBox;
			getSelectionBounds(x0, y0, width0, height0);
			//Coordinates of the line to the center of the image
			xCenter=round(width0/2);
			yCenter=round(height0/2);
			newX0Line=newX0Line-xCenter;
			newY0Line=newY0Line-yCenter;
			newX1Line=newX1Line-xCenter;
			newY1Line=newY1Line-yCenter;
			//print(angle); 
			//print(nbPixToEnlarge);
			//resizeFactor=1;
			
			// Problem with resize function for hyperstacks!!
			//Needs to be fixed!!
			if (resizeFactor!=1){
				getDimensions(widthIm, heightIm, channelsIm, slicesIm, framesIm);
				resizedWidth=round(resizeFactor*width0);
				resizedHeight=round(resizeFactor*height0);
				if (slicesIm==1){
					run("Size...", "width="+resizedWidth+" height="+resizedHeight+" depth="+framesIm+" constrain average interpolation=Bicubic");
				}
				else if (framesIm==1){
						run("Size...", "width="+resizedWidth+" height="+resizedHeight+" depth="+slicesIm+" constrain average interpolation=Bicubic");
				}
				else{
				// run("Size...", "width=120 height=107 depth=5 time=60 constrain average interpolation=Bicubic");
				run("Size...", "width="+resizedWidth+" height="+resizedHeight+" depth="+slicesIm+" time="+framesIm+" constrain average interpolation=Bicubic");
				}
			}
			
			//run("Rotate... ", "angle="+angle+" grid=1 interpolation=Bilinear enlarge stack"); //Bicubic should work too
			
			run("Rotate... ", "angle="+angle+" grid=0 interpolation=Bicubic stack"); //Bicubic or Bilinear should work too
			//Coordinates of the line to the center of the cropped rotated image
			if (drawMembrane){
				angleRad=angle*PI/180;
				rotatedX0Line=round(cos(angleRad)*newX0Line-sin(angleRad)*newY0Line);
				rotatedY0Line=round(sin(angleRad)*newX0Line+cos(angleRad)*newY0Line);
				rotatedX1Line=round(cos(angleRad)*newX1Line-sin(angleRad)*newY1Line);
				rotatedY1Line=round(sin(angleRad)*newX1Line+cos(angleRad)*newY1Line);
				//Coordinates of the line in the rescaled cropped rotated image
				rotatedX0Line=resizeFactor*(rotatedX0Line+xCenter);
				rotatedY0Line=resizeFactor*(rotatedY0Line+yCenter);
				rotatedX1Line=resizeFactor*(rotatedX1Line+xCenter);
				rotatedY1Line=resizeFactor*(rotatedY1Line+yCenter);
				
				// print("X0="+rotatedX0Line);
				// print("Y0="+rotatedY0Line);
				// print("X1="+rotatedX1Line);
				// print("Y1="+rotatedY1Line);
				
				makeLine(rotatedX0Line,rotatedY0Line,rotatedX1Line,rotatedY1Line);
				if (coloredFinalMontage){
					run("RGB Color");
				}
				setColor(0, 0, 255);
				run("Fill", "stack");
				run("Draw","stack");
			}
			
			
			run("Select All");
			run("Enlarge...", "enlarge=-"+(nbPixToEnlarge*resizeFactor)+"  pixel");
			run("Crop");
			
		}
		
	}
	idStraightenCell=getImageID();
	Stack.setPosition(channel, slice, frame);		
	if (measureProfiles){
		getDimensions(sWidth, sHeight, sChannels, sSlices, sFrames);
		profile=newArray(sWidth);
		profilePerAreaUnit=newArray(sWidth);
		diameters=newArray(sWidth);
		xValues=newArray(sWidth);
		for (i=0;i<sWidth;i++){
			curPxDiam=0;
			curInt=0;
			for (j=0;j<sHeight;j++){
				curPix=getPixel(i,j);
				// Get only data above 0 if background is 0, all data otherwise
				if (((multiplyBackgroundBy==0)&&(curPix>multiplyBackgroundBy))||(multiplyBackgroundBy!=0)){ 
					curInt=curInt+curPix;
					curPxDiam=curPxDiam+1;
				}
			}
			xValues[i]=(i-1)*pixelWidth;
			profile[i]=curInt;
			diameters[i]=curPxDiam*pixelHeight;
			if (curPxDiam>0){
				profilePerAreaUnit[i]=curInt/(PI*curPxDiam*pixelHeight*pixelWidth);
			}
			else{
				profilePerAreaUnit[i]=0;
			}
		}
	}
	
	//Remove the zero datapoints (based on diameter value)
	// indexFirstNonZeroDiameter=-1;
	// indexLastNonZeroDiameter=-1;
	// for (i=0;i<sWidth;i++){
	// print(diameters[i]);
	// print(indexFirstNonZeroDiameter);
	// print(indexLastNonZeroDiameter);
	// if ((diameters[i]>0) && indexFirstNonZeroDiameter==-1){
	// indexFirstNonZeroDiameter=i;
	// }
	// else if (indexFirstNonZeroDiameter!=-1 && indexLastNonZeroDiameter==-1 && diameters[i]==0){
	// indexLastNonZeroDiameter=i-1;
	// }
	// }
	// if(indexLastNonZeroDiameter==-1){
	// indexLastNonZeroDiameter=sWidth-1;
	// }
	// lengthNonZeros=indexLastNonZeroDiameter-indexFirstNonZeroDiameter+1;
	// aa
	// xValues0=newArray(lengthNonZeros);
	// diameters0=newArray(lengthNonZeros);
	// profile0=newArray(lengthNonZeros);
	// profilePerAreaUnit0=newArray(lengthNonZeros);
	// for (i=0;i<lengthNonZeros;i++){
	// xValues0[i]=xValues[i+indexFirstNonZeroDiameter];
	// diameters0[i]=diameters[i+indexFirstNonZeroDiameter];
	// profile0[i]=profile[i+indexFirstNonZeroDiameter];
	// profilePerAreaUnit0[i]=profilePerAreaUnit[i+indexFirstNonZeroDiameter];		
	// }
	// xValues=xValues0;
	// diameters=diameters0;
	// profile=profile0;
	// profilePerAreaUnit=profilePerAreaUnit0;
	
	idImageProfile=getImageID();
	if (!makeMontageCellProfile){
		close();
	}
	selectImage(idCopy);
	close();
	return idImageProfile;
}

function array2Text(array, lineLabel){
	txt=lineLabel+"\t";
	size = array.length;
	for (i=1;i<size;i++){
		txt=txt+array[i]+"\t";
	}
	return txt;
}

function ceil(a){
	if (floor(a)==a){
		return a;	
	}
	else{
		return floor(a)+1;
	}
}
